@@ -1,11 +1,12 @@
 """ Functions to compute metrics on an index """
-import os
-from typing import Dict, Union, Optional, List
+import fsspec
+from typing import Dict, Union, Optional
 
 import numpy as np
 import faiss
 
-from autofaiss.readers.embeddings_iterators import read_embeddings
+from embedding_reader import EmbeddingReader
+
 from autofaiss.indices.index_utils import get_index_size, search_speed_test
 from autofaiss.indices.memory_efficient_flat_index import MemEfficientFlatIndex
 from autofaiss.metrics.recalls import one_recall_at_r, r_recall_at_r
@@ -15,10 +16,8 @@ from autofaiss.utils.decorators import Timeit
 
 
 def compute_fast_metrics(
-    embeddings_path: Union[np.ndarray, str, List[str]],
+    embedding_reader: Union[np.ndarray, EmbeddingReader],
     index: faiss.Index,
-    file_format: str = "npy",
-    embedding_column_name: str = "embeddings",
     omp_threads: Optional[int] = None,
     query_max: Optional[int] = 1000,
 ) -> Dict[str, Union[str, int, float]]:
@@ -28,17 +27,12 @@ def compute_fast_metrics(
     size_bytes = get_index_size(index)
     infos["size in bytes"] = size_bytes
 
-    if isinstance(embeddings_path, (list, str)):
+    if isinstance(embedding_reader, EmbeddingReader):
         # pylint: disable=bare-except
-        query_embeddings, _ = next(
-            read_embeddings(
-                embeddings_path, file_format=file_format, embedding_column_name=embedding_column_name, verbose=False
-            )
-        )
+        query_embeddings, _ = next(embedding_reader(start=0, end=query_max, batch_size=query_max))
     else:
-        query_embeddings = embeddings_path
+        query_embeddings = embedding_reader[:query_max]
 
-    query_embeddings = query_embeddings[:query_max]
     if omp_threads:
         faiss.omp_set_num_threads(1)
     speeds_ms = search_speed_test(index, query_embeddings, ksearch=40, timout_s=10.0)
@@ -62,7 +56,7 @@ def compute_fast_metrics(
 
 
 def compute_medium_metrics(
-    embeddings_path: Union[np.ndarray, str],
+    embedding_reader: Union[np.ndarray, EmbeddingReader],
     index: faiss.Index,
     memory_available: Union[str, float],
     ground_truth: Optional[np.ndarray] = None,
@@ -72,37 +66,31 @@ def compute_medium_metrics(
 
     nb_test_points = 500
 
-    if isinstance(embeddings_path, str):
-        # pylint: disable=bare-except
-        embedding_block, _ = next(read_embeddings(embeddings_path, verbose=False))
-
-        if embedding_block.shape[0] < nb_test_points:
-            # pylint: disable=bare-except
-            embedding_block, _ = next(read_embeddings(embeddings_path, batch_size=nb_test_points, verbose=False))
-
-        query_embeddings = embedding_block[:nb_test_points]
+    if isinstance(embedding_reader, EmbeddingReader):
+        query_embeddings, _ = next(embedding_reader(start=0, end=nb_test_points, batch_size=nb_test_points))
     else:
-        embedding_block = embeddings_path
+        embedding_block = embedding_reader
 
         query_embeddings = embedding_block[:nb_test_points]
 
     if ground_truth is None:
-        if isinstance(embeddings_path, str):
-            ground_truth_path = f"{embeddings_path}/small_ground_truth_test.gt"
-            if not os.path.exists(ground_truth_path):
+        if isinstance(embedding_reader, EmbeddingReader):
+            ground_truth_path = f"{embedding_reader.embeddings_folder}/small_ground_truth_test.gt"
+            fs, path = fsspec.core.url_to_fs(ground_truth_path)
+            if not fs.exists(path):
 
                 with Timeit("-> Compute small ground truth", indent=1):
 
                     ground_truth = get_ground_truth(
-                        index.metric_type, embeddings_path, query_embeddings, memory_available
+                        index.metric_type, embedding_reader, query_embeddings, memory_available
                     )
 
-                    with open(ground_truth_path, "wb") as gt_file:
+                    with fs.open(path, "wb") as gt_file:
                         np.save(gt_file, ground_truth)
 
             else:
                 with Timeit("-> Load small ground truth", indent=1):
-                    with open(ground_truth_path, "rb") as gt_file:
+                    with fs.open(path, "rb") as gt_file:
                         ground_truth = np.load(gt_file)
         else:
             ground_truth = get_ground_truth(index.metric_type, embedding_block, query_embeddings, memory_available)
@@ -123,7 +111,7 @@ def compute_medium_metrics(
 
 def get_ground_truth(
     faiss_metric_type: int,
-    embeddings_path: Union[np.ndarray, str],
+    embedding_reader: Union[np.ndarray, EmbeddingReader],
     query_embeddings: np.ndarray,
     memory_available: Union[str, float],
 ):
@@ -131,18 +119,18 @@ def get_ground_truth(
 
     dim = query_embeddings.shape[-1]
 
-    if isinstance(embeddings_path, str):
+    if isinstance(embedding_reader, EmbeddingReader):
         perfect_index = MemEfficientFlatIndex(dim, faiss_metric_type)
-        perfect_index.add_files(embeddings_path)
+        perfect_index.add_files(embedding_reader)
     else:
         perfect_index = faiss.IndexFlat(dim, faiss_metric_type)
-        perfect_index.add(embeddings_path.astype("float32"))  # pylint: disable= no-value-for-parameter
+        perfect_index.add(embedding_reader.astype("float32"))  # pylint: disable= no-value-for-parameter
 
     memory_available = cast_memory_to_bytes(memory_available) if isinstance(memory_available, str) else memory_available
 
     batch_size = int(min(memory_available, 10 ** 9) / (dim * 4))  # at most 1GB of memory
 
-    if isinstance(embeddings_path, str):
+    if isinstance(embedding_reader, EmbeddingReader):
         _, ground_truth = perfect_index.search_files(query_embeddings, k=40, batch_size=batch_size)
     else:
         _, ground_truth = perfect_index.search(query_embeddings, k=40)
